#include <Window.h>
#include <StringView.h>
#include <Button.h>
#include "Nes.h"
#include "ccimport.h"

const uint32 IDC_EDIT1 = 'ide1';
const uint32 IDC_EDIT2 = 'ide2';
const uint32 IDC_EDIT3 = 'ide3';
const uint32 IDC_EDIT4 = 'ide4';
const uint32 IDC_CLEAR = 'clea';

Nes::Nes(BRect frame, BWindow *parent, uint32 msg)
	: BWindow(frame, "Nes", B_TITLED_WINDOW, B_NOT_ZOOMABLE | B_NOT_RESIZABLE),
	qmsg(msg),
	handle(parent),
	m_ngg(""),
	m_nhx1(""),
	m_nhx2(""),
	m_nhx3("")
{
	BView *view = new BView(Bounds(), "", B_FOLLOW_ALL, B_WILL_DRAW);
	view->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	AddChild(view);
	
	gg1 = new CEdit(BRect(10, 10, 130, 60), "gg1", "GG Code", "", 0);
	gg1->SetModificationMessage(new BMessage(IDC_EDIT1));
	gg1->SetDivider(50);
	gg1->SetMaxChars(8);
	view->AddChild(gg1);
	
	hc1 = new CEdit(BRect(10, 40, 100, 60), "hc1", "Hex Code", "", 0);
	hc1->SetModificationMessage(new BMessage(IDC_EDIT2));
	hc1->SetDivider(50);
	hc1->SetMaxChars(4);
	view->AddChild(hc1);
	
	BStringView *sv1 = new BStringView(BRect(102, 35, 108, 55), "", "?");
	view->AddChild(sv1);
	
	hc2 = new CEdit(BRect(110, 40, 140, 60), "hc2", "", "", 0);
	hc2->SetModificationMessage(new BMessage(IDC_EDIT3));
	hc2->SetDivider(0);
	hc2->SetMaxChars(2);
	view->AddChild(hc2);
	
	BStringView *sv2 = new BStringView(BRect(142, 35, 148, 55), "", " :");
	view->AddChild(sv2);
	
	hc3 = new CEdit(BRect(150, 40, 180, 60), "hc3", "", "", 0);
	hc3->SetModificationMessage(new BMessage(IDC_EDIT4));
	hc3->SetDivider(0);
	hc3->SetMaxChars(2);
	view->AddChild(hc3);
	
	view->AddChild(new BButton(BRect(200, 10, 250, 30), "", "Clear", new BMessage(IDC_CLEAR)));
	view->AddChild(new BButton(BRect(200, 40, 250, 60), "", "Close", new BMessage(B_QUIT_REQUESTED)));
}

void Nes::MessageReceived(BMessage *m)
{
	switch(m->what) {
		case IDC_EDIT1:
			OnChangeEdit1();
		break;
		case IDC_EDIT2:
			OnChangeEdit2();
		break;
		case IDC_EDIT3:
			OnChangeEdit3();
		break;
		case IDC_EDIT4:
			OnChangeEdit4();
		break;
		case IDC_CLEAR:
			gg1->SetText("");
			hc1->SetText("");
			hc2->SetText("");
			hc3->SetText("");
		break;
		default:
			BWindow::MessageReceived(m);
		break;
	}
}
#include <stdio.h>
void Nes::OnChangeEdit1() 
{
	CString hresult;
	
	gg1->GetDlgItemText(m_ngg);
	if (!(m_ngg.GetLength()==6 || m_ngg.GetLength()==8))
	{
	}
	else
	{
		if (m_ngg.GetLength()==6)
		{
			hresult = NES6GGtoHex(m_ngg);
			
			hc1->SetDlgItemText(hresult.Left(4));
			hc2->SetDlgItemText("");
			hc3->SetDlgItemText(hresult.Mid(4,2));
		}
		else
		{
			hresult = NES8GGtoHex(m_ngg);
			hc1->SetDlgItemText(hresult.Left(4));
			hc2->SetDlgItemText(hresult.Mid(4,2));
			hc3->SetDlgItemText(hresult.Mid(6,2));
		}
	}
}

void Nes::OnChangeEdit2() 
{
	CString gresult;

	hc1->GetDlgItemText(m_nhx1);
	hc2->GetDlgItemText(m_nhx2);
	hc3->GetDlgItemText(m_nhx3);
	
	if (!(m_nhx1.GetLength()==4 && m_nhx3.GetLength()==2))
	{
		if (m_nhx1.GetLength()==4)
			hc2->SetFocus();
		return;
	}
	if (m_nhx2.GetLength()==1)
	{
		return;
	}
	if (m_nhx2.GetLength()==0)
		gresult = NES6HextoGG(m_nhx1 + m_nhx3);
	else
		gresult = NES8HextoGG(m_nhx1 + m_nhx2 + m_nhx3);
	
	gg1->SetDlgItemText(gresult);
}

void Nes::OnChangeEdit3() 
{
	CString gresult;

	hc1->GetDlgItemText(m_nhx1);
	hc2->GetDlgItemText(m_nhx2);
	hc3->GetDlgItemText(m_nhx3);
	
	if (!(m_nhx1.GetLength()==4 && m_nhx3.GetLength()==2))
	{
		if (m_nhx2.GetLength()==2)
			hc3->SetFocus();
		return;
	}
	if (m_nhx2.GetLength()==1)
	{
		return;
	}
	if (m_nhx2.GetLength()==0)
		gresult = NES6HextoGG(m_nhx1 + m_nhx3);
	else
		gresult = NES8HextoGG(m_nhx1 + m_nhx2 + m_nhx3);

	gg1->SetDlgItemText(gresult);
}

void Nes::OnChangeEdit4() 
{
	CString gresult;
		
	hc1->GetDlgItemText(m_nhx1);
	hc2->GetDlgItemText(m_nhx2);
	hc3->GetDlgItemText(m_nhx3);

	if (!(m_nhx1.GetLength()==4 && m_nhx3.GetLength()==2))
	{
		return;
	}
	if (m_nhx2.GetLength()==1)
	{
		return;
	}
	if (m_nhx2.GetLength()==0)
		gresult = NES6HextoGG(m_nhx1 + m_nhx3);
	else
		gresult = NES8HextoGG(m_nhx1 + m_nhx2 + m_nhx3);

	gg1->SetDlgItemText(gresult);
}

void Nes::Quit()
{
	handle->PostMessage(qmsg);
	handle->Show();
	BWindow::Quit();
}