#include <Window.h>
#include <StringView.h>
#include <TextView.h>
#include <Button.h>
#include "CEdit.h"
#include "CString.h"
#include "Snes.h"
#include "ccimport.h"
#include <stdio.h>

const uint32 IDC_EDIT1 = 'ide1';
const uint32 IDC_EDIT2 = 'ide2';
const uint32 IDC_EDIT3 = 'ide3';
const uint32 IDC_EDIT4 = 'ide4';
const uint32 IDC_CLEAR = 'clea';

Snes::Snes(BRect frame, BWindow *parent, uint32 msg)
	: BWindow(frame, "Snes", B_TITLED_WINDOW, B_NOT_ZOOMABLE | B_NOT_RESIZABLE),
	m_sgg1(""),
	m_sgg2(""),
	m_shx1(""),
	m_shx2(""),
	qmsg(msg),
	handle(parent)
{
	BView *view = new BView(Bounds(), "", B_FOLLOW_ALL, B_WILL_DRAW);
	view->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	AddChild(view);
	
	gg1 = new CEdit(BRect(10, 10, 100, 60), "gg1", "GG Code", "", 0);
	gg1->SetModificationMessage(new BMessage(IDC_EDIT1));
	gg1->SetDivider(50);
	gg1->SetMaxChars(4);
	view->AddChild(gg1);
	
	BStringView *sv = new BStringView(BRect(100, 15, 110, 25), "", " --");
	view->AddChild(sv);
	
	gg2 = new CEdit(BRect(110, 10, 150, 60), "gg2", "", "", 0);
	gg2->SetModificationMessage(new BMessage(IDC_EDIT2));
	gg2->SetDivider(0);
	gg2->SetMaxChars(4);
	view->AddChild(gg2);
	
	hc1 = new CEdit(BRect(10, 40, 110, 60), "hc1", "Hex Code", "", 0);
	hc1->SetModificationMessage(new BMessage(IDC_EDIT3));
	hc1->SetMaxChars(6);
	view->AddChild(hc1);
	BStringView *sv1 = new BStringView(BRect(112, 35, 120, 55), "", " :");
	view->AddChild(sv1);
	
	hc2 = new CEdit(BRect(120, 40, 145, 60), "hc2", "", "", 0);
	hc2->SetModificationMessage(new BMessage(IDC_EDIT4));
	hc2->SetDivider(0);
	hc2->SetMaxChars(2);
	view->AddChild(hc2);
	
	BRect rect(10, 70, 60, 60);
	BButton *show32 = new BButton(rect, "", "Show 32k", new BMessage('sh32'));
	view->AddChild(show32);
	
	rect.Set(70, 70, 120, 60);
	BButton *show64 = new BButton(rect, "", "Show 64k", new BMessage('sh64'));
	view->AddChild(show64);
	
	rect.Set(190, 70, 240, 60);
	BButton *quit = new BButton(rect, "", "Close", new BMessage(B_QUIT_REQUESTED));
	view->AddChild(quit);
	
	rect.Set(130, 70, 180, 60);
	BButton *clear = new BButton(rect, "", "Clear", new BMessage(IDC_CLEAR));
	view->AddChild(clear);
	
	tview = new BTextView(BRect(160, 10, 400, 40), "", BRect(0, 0, 120, 40), B_FOLLOW_NONE, B_WILL_DRAW);
	tview->SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	tview->MakeEditable(false);
	tview->MakeSelectable(false);
	view->AddChild(tview);
	tview->SetText("No messages.");
}

void Snes::MessageReceived(BMessage *m)
{
	switch(m->what) {
		case IDC_EDIT1:
			OnChangeEdit1();
		break;
		case IDC_EDIT2:
			OnChangeEdit2();
		break;
		case IDC_EDIT3:
			OnChangeEdit3();
		break;
		case IDC_EDIT4:
			OnChangeEdit4();
		break;
		case 'sh32':
			OnButton1();
		break;
		case 'sh64':
			OnButton2();
		break;
		case IDC_CLEAR:
			gg1->SetText("");
			gg2->SetText("");
			hc1->SetText("");
			hc2->SetText("");
			tview->SetText("No messages.");
		break;
		default:
			BWindow::MessageReceived(m);
		break;
	}
}

static CString CheckAddr(CString scode);

void Snes::OnChangeEdit1() 
{
	CString hresult;
	int i;

	gg1->GetDlgItemText(m_sgg1);
	gg2->GetDlgItemText(m_sgg2);
	
	if (!(m_sgg1.GetLength()==4 && m_sgg2.GetLength()==4))
	{
		if (m_sgg1.GetLength()==4)
			gg2->SetFocus();
		return;
	}
	hresult = SNESGGtoHex(m_sgg1 + m_sgg2);
	m_shx1="      ";
	m_shx2="  ";
	for (i=0;i<=5;i++)
	{
		m_shx1.SetAt(i,hresult[i]);
	}
	for (i=0;i<=1;i++)
	{
		m_shx2.SetAt(i,hresult[i+6]);
	}
	hc1->SetDlgItemText(m_shx1);
	hc2->SetDlgItemText(m_shx2);
	SetWindowText(CheckAddr(m_shx1 + m_shx2));
}

void Snes::OnChangeEdit2() 
{
	CString hresult;
	int i;

	gg1->GetDlgItemText(m_sgg1);
	gg2->GetDlgItemText(m_sgg2);
	
	if (!(m_sgg1.GetLength()==4 && m_sgg2.GetLength()==4))
	{
		return;
	}
	hresult = SNESGGtoHex(m_sgg1 + m_sgg2);
	m_shx1="      ";
	m_shx2="  ";
	for (i=0;i<=5;i++)
	{
		m_shx1.SetAt(i,hresult[i]);
	}
	for (i=0;i<=1;i++)
	{
		m_shx2.SetAt(i,hresult[i+6]);
	}
	hc1->SetDlgItemText(m_shx1);
	hc2->SetDlgItemText(m_shx2);
	SetWindowText(CheckAddr(m_shx1 + m_shx2));
}

void Snes::OnChangeEdit3() 
{
	CString gresult;
	int i;
	
	hc1->GetDlgItemText(m_shx1);
	hc2->GetDlgItemText(m_shx2);
	
	if (!(m_shx1.GetLength()==6 && m_shx2.GetLength()==2))
	{
		if (m_shx1.GetLength()==6)
			hc2->SetFocus();
		return;
	}
	gresult = SNESHextoGG(m_shx1 + m_shx2);
	m_sgg1="    ";
	m_sgg2="    ";
	for (i=0;i<=3;i++)
	{
		m_sgg1.SetAt(i,gresult[i]);
		m_sgg2.SetAt(i,gresult[i+4]);
	}
	gg1->SetDlgItemText(m_sgg1);
	gg2->SetDlgItemText(m_sgg2);
	SetWindowText(CheckAddr(m_shx1 + m_shx2));
}

void Snes::OnChangeEdit4() 
{
	CString gresult;
	int i;

	hc1->GetDlgItemText(m_shx1);
	hc2->GetDlgItemText(m_shx2);
	
	if (!(m_shx1.GetLength()==6 && m_shx2.GetLength()==2))
	{
		return;
	}
	gresult = SNESHextoGG(m_shx1 + m_shx2);
	m_sgg1="    ";
	m_sgg2="    ";
	for (i=0;i<=3;i++)
	{
		m_sgg1.SetAt(i,gresult[i]);
		m_sgg2.SetAt(i,gresult[i+4]);
	}
	gg1->SetDlgItemText(m_sgg1);
	gg2->SetDlgItemText(m_sgg2);
	SetWindowText(CheckAddr(m_shx1 + m_shx2));
}

void Snes::OnButton1() 
{
	//CM: Heh, the only buttons anywhere...
	//You know, I think the formula actually works this time!
	CString gcode="        ";
	CString hcode="        ";
	int i;
	
	hc1->GetDlgItemText(m_shx1);
	hc1->GetDlgItemText(m_shx2);

	if (m_shx1.GetLength() + m_shx2.GetLength() < 8)
	{
		SetWindowText("Hex code must be complete.");
		return;
	}
	for (i=0;i<=5;i++)
	{
		if (chtoi(m_shx1[i])==-1)
		{
			SetWindowText("Error in hex code.");
			return;
		}
	}
	for (i=0;i<=1;i++)
	{
		if (chtoi(m_shx2[i])==-1)
		{
			SetWindowText("Error in hex code.");
			return;
		}
	}
	if (m_shx1[0] < '4' || (m_shx1[0] >= '8' && m_shx1[0] < 'C'))
	{
		SetWindowText("Code is already in 32K range.");
		return;
	}

	hcode=m_shx1+m_shx2;
	hcode=SNES64to32(hcode);
	gcode=SNESHextoGG(hcode);
	hcode=hcode.Left(6)+":"+hcode.Right(2);
	gcode=gcode.Left(4)+"-"+gcode.Right(4);
	SetWindowText(gcode+"\n"+hcode);
}

void Snes::OnButton2() 
{
	CString gcode="        ";
	CString hcode="        ";
	int i;
	
	hc1->GetDlgItemText(m_shx1);
	hc2->GetDlgItemText(m_shx2);

	if (m_shx1.GetLength() + m_shx2.GetLength() < 8)
	{
		SetWindowText("Hex code must be complete.");
		return;
	}
	for (i=0;i<=5;i++)
	{
		if (chtoi(m_shx1[i])==-1)
		{
			SetWindowText("Error in hex code.");
			return;
		}
	}
	for (i=0;i<=1;i++)
	{
		if (chtoi(m_shx2[i])==-1)
		{
			SetWindowText("Error in hex code.");
			return;
		}
	}
	if ((m_shx1[0] >= '4' && m_shx1[0] < '8') || m_shx1[0] >= 'C')
	{
		SetWindowText("Code is already in 64K range.");
		return;
	}

	hcode=m_shx1+m_shx2;
	hcode=SNES32to64(hcode);
	gcode=SNESHextoGG(hcode);
	hcode=hcode.Left(6)+":"+hcode.Right(2);
	gcode=gcode.Left(4)+"-"+gcode.Right(4);
	SetWindowText(gcode+"\n"+hcode);
}

void Snes::SetWindowText(CString string)
{
	tview->SetText(string.String());
}

void Snes::Quit()
{
	handle->PostMessage(qmsg);
	handle->Show();
	BWindow::Quit();
}

static CString CheckAddr(CString scode)
{	//This'll keep those 7E's out... Why doesn't anyone believe me?
	if (scode[0]=='7')
		return "Critical: Address is not within ROM range.";
	if ((scode[0]>='0' && scode[0]<='3') && (scode[2] >= '0' && scode[2] <= '7'))
		return "Critical: Address is not within ROM range.";
	if ((scode[0]>='8' && scode[0]<='9') && (scode[2] >= '0' && scode[2] <= '7'))
		return "Critical: Address is not within ROM range.";
	if ((scode[0]>='A' && scode[0]<='B') && (scode[2] >= '0' && scode[2] <= '7'))
		return "Critical: Address is not within ROM range.";
	if ((scode[0]>='4' && scode[0]<='6') && (scode[2] >= '0' && scode[2] <= '7'))
		return "Minor: Address is HiROM only.";
	if ((scode[0]>='C' && scode[0]<='E') && (scode[2] >= '0' && scode[2] <= '7'))
		return "Minor: Address is HiROM only.";
	if (scode[0]=='F')
		return "Minor: Address is HiROM only.";
	return "No messages.";
}